unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, VCLTee.TeeTools,
  VCLTee.TeEngine, VCLTee.Series, Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart;

type
  TForm1 = class(TForm)
    Chart1: TChart;
    Series1: TLineSeries;
    lintlRightSide: TColorLineTool;
    anntoolRightside: TAnnotationTool;
    imgChart: TImage;
    procedure Chart1BeforeDrawSeries(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateAnnotationPosition(aChart: TChart);
    procedure UpdateImage;
  protected
    procedure Resize; override;
  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;
  end;

var
  Form1: TForm1;

implementation

uses
  math, VCLTee.TeCanvas;

{$R *.dfm}

const
  CLR_RIGHT_SIDE = clGreen;
  NUM_POINTS = 10;
  RIGHT_SIDE_DESCRIP = 'Right Side'#13'Of Chart';

procedure AssignChartToGraphic(aChart: TChart; const aRect: TRect; aGraphic: TGraphic; aTransparent: boolean = false);
var
  lHoldShadow: boolean;
  I: Integer;
  lBoolArray: array of boolean;
  lHoldBevel: TBevelCut;
  lHoldColor: TColor;
  lHoldGradientVisible: boolean;
  lMetafile: TMetafile;
  lBmp: TBitmap;
  lCanvasClass: TCanvas3DClass;
begin
  lCanvasClass := TCanvas3DClass(aChart.Canvas.ClassType);
  if lCanvasClass <> TTeeCanvas3D then
    aChart.Canvas := TTeeCanvas3D.Create;

  lHoldGradientVisible := aChart.Gradient.Visible;
  lHoldColor := aChart.Color;
  lHoldBevel := aChart.BevelOuter;
  lHoldShadow := aChart.Shadow.Visible;

  try
    SetLength(lBoolArray, aChart.SeriesCount);
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
      begin
        lBoolArray[I] := TPieSeries(aChart.Series[I]).Circled;
        TPieSeries(aChart.Series[I]).Circled := True;
      end
      else
        lBoolArray[I] := True;


    aChart.Gradient.Visible  := false;
    if not aTransparent then
      aChart.Color := clWhite
    else
      aChart.Color := clNone; //clWhite;
    aChart.BevelOuter := bvNone;
    aChart.BevelInner := bvNone;
    aChart.Shadow.Visible := False;

    if aGraphic is TMetafile then
    begin
      lMetafile := aChart.TeeCreateMetafile(false, aRect);
      try
        aGraphic.Assign(lMetafile);
      finally
        lMetafile.Free;
      end;
    end
    else
    begin
      lBmp := aChart.TeeCreateBitmap(clWhite, aRect);
      try
        aGraphic.Assign(lBmp);
      finally
        lBmp.Free;
      end;
    end;
  finally
    aChart.Shadow.Visible := lHoldShadow;
    aChart.Gradient.Visible := lHoldGradientVisible;
    aChart.Color := lHoldColor;
    aChart.BevelOuter := lHoldBevel;
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
        TPieSeries(aChart.Series[I]).Circled := lBoolArray[I];
    if (lCanvasClass <> aChart.Canvas.ClassType) then
      aChart.Canvas := lCanvasClass.Create;
  end;
end;

{ TForm1 }

procedure ConfigChart(const aChart: TChart);
var
 lAxis: TChartAxis;
begin
  aChart.Title.Font.Size := 12;
  aChart.Title.Font.Style := [fsBold];
  aChart.Title.Font.Color := clNavy;
  aChart.Shadow.Visible := false;

  lAxis := aChart.LeftAxis;

  lAxis.Title.Font.Size := 10;
  lAxis.Title.Font.Style := [fsBold];

  lAxis.LabelsFont.Size := 9;
  lAxis.LabelsSeparation := 100;
  lAxis.Axis.Width := 1;
  lAxis.Ticks.Visible := False;
  lAxis.MinorTicks.Visible := False;
  lAxis.TicksInner.Visible := False;
  lAxis.Grid.Color := clSilver;
  lAxis.Grid.Style := psDot;

  lAxis := aChart.BottomAxis;

  lAxis.Title.Font.Size := 10;
  lAxis.Title.Font.Style := [fsBold];
  lAxis.Axis.Width := 1;
  lAxis.Grid.Visible := False;
  lAxis.MinorTicks.Visible := False;
  lAxis.TicksInner.Visible := False;

  lAxis.LabelsFont.Size := 9;
  lAxis.LabelsSeparation := 50;

  aChart.Color := clWhite;
  aChart.RightAxis.Visible := False;
  aChart.TopAxis.Visible := False;
  aChart.BackWall.Visible := False;

  aChart.LeftWall.Color := clWhite;

  aChart.Legend.Font.Size := 9;
  aChart.Legend.Symbol.Squared := True;
  //aChart.Legend.Symbol.DefaultPen := False;
  aChart.Legend.Symbol.Pen.Visible := False;
  aChart.Legend.Symbol.Shadow.Visible := False;
  aChart.Legend.Gradient.Visible := False;
  aChart.Legend.Shadow.Visible := False;
  aChart.Legend.Frame.Visible := False;

  aChart.View3D := False;
  aChart.Gradient.Visible := False;
  aChart.Zoom.Allow := false;
end;

constructor TForm1.Create(aOwner: TComponent);
begin
  inherited;

  anntoolRightSide.Text := RIGHT_SIDE_DESCRIP;
  anntoolRightSide.Shape.Frame.Color := CLR_RIGHT_SIDE;
  anntoolRightSide.Shape.ShapeStyle := fosRoundRectangle;

  lintlRightSide.Pen.Color := CLR_RIGHT_SIDE;

  ConfigChart(Chart1);


  Series1.FillSampleValues(NUM_POINTS);

  lintlRightSide.Value := NUM_POINTS - 1;
end;

procedure TForm1.Resize;
begin
  inherited;
  imgChart.Width := Width div 3;
  UpdateImage;
end;

procedure TForm1.UpdateImage;
  procedure  SetAnnotationShadow(const aSize: integer);
  var
    I: integer;
    lAnnTool: TAnnotationTool;
  begin
    for I := 0 to Chart1.Tools.Count-1 do
      if Chart1.Tools[I] is TAnnotationTool then
      begin
        lAnnTool := TAnnotationTool(Chart1.Tools[I]);
        lAnnTool.Shape.Shadow.Visible := aSize > 0;
        if aSize > 0 then
        begin
          lAnnTool.Shape.Shadow.HorizSize := aSize;
          lAnnTool.Shape.Shadow.VertSize := aSize;
        end;
      end;
  end;
var
  lRect: TRect;
  lGraphic: TGraphic;
begin
  lRect := Rect(1, 1, imgChart.Width, imgChart.Height);
  lGraphic := TBitmap.Create;
  try
    SetAnnotationShadow(0);
    lGraphic.SetSize(imgChart.Width, imgChart.Height);
    AssignChartToGraphic(Chart1, lRect, lGraphic);

    imgChart.Picture.Graphic := lGraphic;
  finally
    lGraphic.Free;
    SetAnnotationShadow(3);
  end;
end;
procedure TForm1.Chart1BeforeDrawSeries(Sender: TObject);
begin
  UpdateAnnotationPosition(Sender as TChart);
end;

procedure TForm1.UpdateAnnotationPosition(aChart: TChart);
var
  lChartTool: TTeeCustomTool;
  lIdx: Integer;

  procedure SetLeftPos(aMiddle: double; aAnnTool: TAnnotationTool);
  var
    lLeft: Integer;
    lR: TRect;
    lAnnWidth: Integer;
  begin
    if not aAnnTool.Active then
      exit;

    lLeft :=  aChart.BottomAxis.CalcXPosValue(aMiddle);
    lR := aAnnTool.Bounds;
    lAnnWidth := lR.Right- lR.Left + 1;
    aAnnTool.Left := lLeft - (Max(0, lAnnWidth) div 2);
    if aAnnTool.Left + aAnnTool.Shape.Width > aChart.Width then
      aAnnTool.Left := aChart.Width - aAnnTool.Shape.Width;

    aAnnTool.Shape.Top := 30;
  end;

  function GetAnnotationTool(aTitle: String): TAnnotationTool;
  var
    I: Integer;
    lComp: TComponent;
  begin
    for I := 0 to ComponentCount - 1 do    // Iterate
    begin
      lComp := Components[I];
      if (lComp is TAnnotationTool) and
        (TAnnotationTool(lComp).Text = aTitle) then
        exit(TAnnotationTool(lComp))
    end;
    result := nil;
  end;

  function GetLineTool(aColor: TColor): TColorLineTool;
  var
    I: Integer;
    lComp: TComponent;
  begin
    for I := 0 to ComponentCount - 1 do    // Iterate
    begin
      lComp := Components[I];
      if (lComp is TColorLineTool) and
        (TColorLineTool(lComp).Pen.Color = aColor) then
        exit(TColorLineTool(lComp));
    end;
    result := nil;
  end;

var
  lAnnotate: TAnnotationTool;
  lLineTool: TColorLineTool;
  lRects: array of TRect;
begin
  SetLength(lRects, 1);
  lLineTool := GetLineTool(CLR_RIGHT_SIDE);
  lAnnotate := GetAnnotationTool(RIGHT_SIDE_DESCRIP);
  SetLeftPos(lLineTool.Value, lAnnotate);
end;


end.
